@@ -26,7 +26,7 @@
 function(X, Z, XX=NULL, bprior="bflat", corr="expsep", tree=c(0.25,2,10),
          gamma=c(10,0.2,0.7), BTE=c(2000,7000,2), R=1, m0r1=FALSE,
          linburn=FALSE, pred.n=TRUE, ds2x=FALSE, ego=FALSE, nu=0.5,
-         verb=1)
+         traces=FALSE, verb=1)
 {
   n <- dim(X)[1]
   if(is.null(n)) { n <- length(X); X <- matrix(X, nrow=n); d <- 1 }
@@ -37,6 +37,6 @@ function(X, Z, XX=NULL, bprior="bflat", corr="expsep", tree=c(0.25,2,10),
   params$tree <- tree
   params$gamma <- gamma
   if(corr == "matern"){ params$nu <- nu }
-  return(tgp(X,Z,XX,BTE,R,m0r1,linburn,params,pred.n,ds2x,ego,verb))
+  return(tgp(X,Z,XX,BTE,R,m0r1,linburn,params,pred.n,ds2x,ego,traces,verb))
 }
 
