@@ -53,10 +53,15 @@ using namespace std;
  * constructor function
  */
 
-Exp::Exp(unsigned int col, Gp_Prior *gp_prior)
-  : Corr(col, gp_prior)
+Exp::Exp(unsigned int col, Base_Prior *base_prior)
+  : Corr(col, base_prior)
 {
-  assert(gp_prior->CorrPrior()->CorrModel() == EXP);
+  assert(base_prior->BaseModel() == GP);
+  prior = ((Gp_Prior*) base_prior)->CorrPrior();
+  assert(prior);
+  nug = prior->Nug();
+
+  assert( ((Gp_Prior*) base_prior)->CorrPrior()->CorrModel() == EXP);
   d = ((Exp_Prior*) prior)->D();
   xDISTx = NULL;
   nd = 0;
@@ -82,7 +87,7 @@ Corr& Exp::operator=(const Corr &c)
   nug = e->nug;
   dreject = e->dreject;
   assert(prior->CorrModel() == EXP);
-  assert(prior == gp_prior->CorrPrior());
+  assert(prior == ((Gp_Prior*) base_prior)->CorrPrior());
   
   /* copy the covariance matrices */
   Cov(e);
@@ -103,6 +108,40 @@ Exp::~Exp(void)
   xDISTx = NULL;
 }
 
+/* 
+ * DrawNug:
+ * 
+ * draw for the nugget; 
+ * rebuilding K, Ki, and marginal params, if necessary 
+ * return true if the correlation matrix has changed; false otherwise
+ */
+
+bool Exp::DrawNug(unsigned int n, double **X, double **F, 
+		  double *Z, double *lambda, 
+		   double **bmu, double **Vb, double tau2, void *state)
+{
+  bool success = false;
+  Gp_Prior *gp_prior = (Gp_Prior*) base_prior;
+
+  /* allocate K_new, Ki_new, Kchol_new */
+  if(! linear) assert(n == this->n);
+  
+  /* randomly reject 1/2 the time, to avoid having to do lots of matrix
+     inversions -- as the nug mixes better than d already */
+  if(runi(state) > 0.5) return false;
+  
+  /* make the draw */
+  double nug_new = 
+    nug_draw_margin(n, col, nug, F, Z, K, log_det_K, *lambda, Vb, K_new, Ki_new, 
+		    Kchol_new, &log_det_K_new, &lambda_new, Vb_new, bmu_new, gp_prior->get_b0(), 
+		    gp_prior->get_Ti(), gp_prior->get_T(), tau2, prior->NugAlpha(), prior->NugBeta(), 
+		    gp_prior->s2Alpha(), gp_prior->s2Beta(), (int) linear, state);
+  
+  /* did we accept the draw? */
+  if(nug_new != nug) { nug = nug_new; success = true; swap_new(Vb, bmu, lambda); }
+  
+  return success;
+}
 
 /*
  * Update: (symmetric)
@@ -173,8 +212,9 @@ int Exp::Draw(unsigned int n, double **F, double **X, double *Z,
   bool lin_new;
   double q_fwd , q_bak, d_new;
 
-  /* sometimes skip this Draw for linear models for speed */
-  if(linear && runi(state) > 0.5) return DrawNug(n, F, Z, lambda, bmu, Vb, tau2, state);
+  /* sometimes skip this Draw for linear models for speed,
+   and only draw the nugget */
+  if(linear && runi(state) > 0.5) return DrawNug(n, X, F, Z, lambda, bmu, Vb, tau2, state);
 
   /* proppose linear or not */
   if(prior->Linear()) lin_new = true;
@@ -185,7 +225,7 @@ int Exp::Draw(unsigned int n, double **F, double **X, double *Z,
     else lin_new = false;
   }
 
-  /* if not linear than compute new distances */
+  /* if not linear then compute new distances */
   /* allocate K_new, Ki_new, Kchol_new */
   if(! lin_new) {
     if(!xDISTx || nd != n)  {
@@ -202,6 +242,8 @@ int Exp::Draw(unsigned int n, double **F, double **X, double *Z,
   if(prior->Linear()) d_new = d;
   else {
     Exp_Prior* ep = (Exp_Prior*) prior;
+    Gp_Prior *gp_prior = (Gp_Prior*) base_prior;
+
     success = 
       d_draw_margin(n, col, d_new, d, F, Z, xDISTx, log_det_K, *lambda, Vb, K_new, 
 		    Ki_new, Kchol_new, &log_det_K_new, &lambda_new, Vb_new, bmu_new,  
@@ -219,7 +261,7 @@ int Exp::Draw(unsigned int n, double **F, double **X, double *Z,
   else if(success == 0) dreject++;
   
   /* draw nugget */
-  bool changed = DrawNug(n, F, Z, lambda, bmu, Vb, tau2, state);
+  bool changed = DrawNug(n, X, F, Z, lambda, bmu, Vb, tau2, state);
   success = success || changed;
   
   return success;
@@ -385,6 +427,24 @@ double Exp::log_Prior(void)
 }
 
 
+/* 
+ * Trace:
+ *
+ * return the current values of the parameters
+ * to this correlation function: nug, d, then linear
+ */
+
+double* Exp::Trace(unsigned int* len)
+{
+  *len = 3;
+  double *trace = new_vector(*len);
+  trace[0] = nug;
+  trace[1] = d;
+  trace[2] = (double) !linear;
+  return trace;
+}
+
+
 /*
  * newCorr:
  *
@@ -394,7 +454,7 @@ double Exp::log_Prior(void)
 
 Corr* Exp_Prior::newCorr(void)
 {
-  return new Exp(col, gp_prior);
+  return new Exp(col, base_prior);
 }
 
 
@@ -648,6 +708,28 @@ double Exp_Prior::log_Prior(double d, bool linear)
   return prob;
 }
 
+/* 
+ * BasePrior:
+ *
+ * return the prior for the Base (eg Gp) model
+ */
+
+Base_Prior* Exp_Prior::BasePrior(void)
+{
+  return base_prior;
+}
+
+
+/*
+ * SetBasePrior:
+ *
+ * set the base_prior field
+ */
+
+void Exp_Prior::SetBasePrior(Base_Prior *base_prior)
+{
+  this->base_prior = base_prior;
+}
 
 /*
  * Print:
