@@ -44,7 +44,7 @@ class ExpSep : public Corr
   double *pb;		/* prob of dimension-wize linearization */
   unsigned int dreject; /* d rejection counter */
  public:
-  ExpSep(unsigned int col, Gp_Prior *prior);
+  ExpSep(unsigned int col, Base_Prior *base_prior);
   virtual Corr& operator=(const Corr &c);
   virtual ~ExpSep(void);
   virtual void Update(unsigned int n1, unsigned int n2, double **K, double **X, double **XX);
@@ -57,6 +57,11 @@ class ExpSep : public Corr
   virtual char* State(void);
    virtual unsigned int sum_b(void);
   virtual void ToggleLinear(void);
+  virtual bool DrawNug(unsigned int n, double **X, double **F, double *Z,
+		       double *lambda, double **bmu, 
+		       double **Vb, double tau2, void *state);
+  virtual double* Trace(unsigned int* len);
+
   void get_delta_d(ExpSep* c1, ExpSep* c2, void *state);
   void propose_new_d(ExpSep* c1, ExpSep* c2, void *state);
   bool propose_new_d(double* d_new, int * b_new, double *pb_new, 
@@ -95,6 +100,9 @@ class ExpSep_Prior : public Corr_Prior
   virtual void Draw(Corr **corr, unsigned int howmany, void *state);
   virtual Corr* newCorr(void);
   virtual void Print(FILE *outfile);
+  virtual Base_Prior* BasePrior(void);
+  virtual void SetBasePrior(Base_Prior *base_prior);
+
 
   void draw_d_from_prior(double *d_new, void *state);
   double* D(void);
